@@ -1,6 +1,7 @@
-# minitest - a minimal testing framework v0.0.2 --------------------------------
+# minitest - a minimal testing framework v0.0.4 --------------------------------
 test_library <- function(package) library(package = package, character.only = TRUE)
 test_true <- function(x) invisible(isTRUE(x) || {print(x); stop("the above was returned instead of TRUE")})
+test_false <- function(x) invisible(isFALSE(x) || {print(x); stop("the above was returned instead of FALSE")})
 test_null <- function(x) invisible(is.null(x) || {print(x); stop("the above was returned instead of NULL")})
 test_notnull <- function(x) invisible(!is.null(x) || stop("returns NULL when expected to be not NULL"))
 test_zero <- function(x) invisible(x == 0L || {print(x); stop("the above was returned instead of 0L")})
@@ -74,7 +75,7 @@ test_error(n$recv(mode = "int"), "mode")
 test_error(n$recv(mode = "logica"), "mode")
 test_error(n$recv(mode = "charact"), "mode")
 test_error(n$recv(mode = "positrons"), "mode")
-test_true(is_aio(raio <- n1$recv_aio(timeout = 1L)))
+test_true(is_aio(raio <- recv_aio(n$socket, timeout = 1L, cv = substitute())))
 test_print(raio)
 test_class("errorValue", call_aio(raio)$data)
 test_class("errorValue", raio$data)
@@ -113,6 +114,7 @@ test_identical(call_aio(msg), msg)
 test_class("recvAio", msg <- n1$recv_aio(mode = "complex", timeout = 500))
 test_null(stop_aio(msg))
 test_null(stop_aio(n))
+test_false(stop_request(n))
 test_identical(call_aio(msg), msg)
 test_class("errorValue", msg$data)
 test_identical(call_aio(n), n)
@@ -179,11 +181,11 @@ test_equal(n1$socket[state], "closed")
 
 test_class("conditionVariable", cv <- cv())
 test_print(cv)
-test_true(!until(cv, 10L))
-test_true(!until(cv, 10))
-test_true(!until_(cv, 10L))
-test_true(!until_(cv, 10))
-test_true(!until_(cv, "test"))
+test_false(until(cv, 10L))
+test_false(until(cv, 10))
+test_false(until_(cv, 10L))
+test_false(until_(cv, 10))
+test_false(until_(cv, "test"))
 test_zero(cv_reset(cv))
 test_zero(cv_value(cv))
 
@@ -219,10 +221,10 @@ test_zero(call_aio_(csaio)$result)
 test_class("recvAio", craio <- recv_aio(ctx, mode = 8L, timeout = 500))
 test_type("raw", res <- collect_aio(craio))
 test_true(.read_marker(res))
-test_true(!.read_marker("not"))
+test_false(.read_marker("not"))
 test_equal(.read_header("not"), 0L)
 test_type("list", unserialize(res[9:length(res)]))
-test_true(!.mark(FALSE))
+test_false(.mark(FALSE))
 test_zero(req$send("context test", mode ="raw", block = 500))
 test_equal(recv(ctx, mode = "string", block = 500), "context test")
 test_type("integer", req$send(data.frame(), mode = "serial", block = 500))
@@ -230,7 +232,7 @@ test_class("recvAio", msg <- recv_aio(ctx, mode = "serial", timeout = 500))
 test_type("logical", .unresolved(msg))
 test_type("logical", unresolved(msg))
 test_class("data.frame", call_aio(msg)$data)
-test_true(!unresolved(msg))
+test_false(unresolved(msg))
 test_zero(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500))
 test_class("recvAio", msg <- recv_aio(ctx, mode = 6L, timeout = 500))
 test_type("logical", msg[])
@@ -272,6 +274,11 @@ opt(req$socket, "serial") <- cfg
 opt(rep, "serial") <- cfg
 test_zero(send(rep, custom, block = 500))
 test_type("list", recv(req$socket, mode = 1L, block = 500))
+cfg <- serial_config(c("error_env", "string_class"), list(function(x) stop("serialization failue"), function(x) "serialized string"), list(function(x) stop(), function(x) stop()))
+opt(req$socket, "serial") <- cfg
+test_error(send(req$socket, `class<-`(new.env(), "error_env"), block = 500), "serialization failue")
+test_error(send(req$socket, `class<-`(new.env(), "string_class"), block = 500), "string_class")
+opt(req$socket, "serial") <- cfg
 opt(req$socket, "serial") <- list()
 opt(rep, "serial") <- list()
 test_error(serial_config(1L, identity, identity), "must be a character vector")
@@ -294,6 +301,7 @@ test_notnull(cs$data)
 test_true(recv(ctxn, block = 500))
 test_zero(send(ctxn, TRUE, mode = 1L, block = 500))
 test_class("recvAio", cs <- request(.context(req$socket), data = TRUE, timeout = 5, id = TRUE))
+test_false(stop_request(cs))
 test_zero(reap(ctxn))
 test_equal(reap(ctxn), 7L)
 test_zero(pipe_notify(rep, cv, add = TRUE, flag = TRUE))
@@ -325,7 +333,7 @@ test_class("nanoObject", sub <- nano("sub", dial = "inproc://ps", autostart = NA
 test_zero(cv_reset(cv))
 test_zero(pipe_notify(pub$socket, cv, add = TRUE, remove = TRUE))
 test_class("nano", sub$opt(name = "sub:prefnew", value = FALSE))
-test_true(!sub$opt(name = "sub:prefnew"))
+test_false(sub$opt(name = "sub:prefnew"))
 test_error(sub$opt(name = "false", value = 100), "supported")
 test_error(sub$opt(name = "false"), "supported")
 test_error(sub$opt(name = "false", value = list()), "type")
@@ -367,7 +375,7 @@ test_zero(surv$context_close())
 test_zero(resp$context_close())
 test_zero(surv$close())
 test_zero(resp$close())
-test_true(!wait(cv))
+test_false(wait(cv))
 test_class("errorValue", resp$recv())
 
 test_class("nanoObject", poly <- nano(protocol = "poly", listen = "inproc://polytest"))
@@ -432,10 +440,10 @@ test_type("list", ncurl("http://www.cam.ac.uk/", follow = FALSE, response = "dat
 test_type("list", ncurl("http://www.cam.ac.uk/", follow = TRUE))
 test_type("list", ncurl("https://postman-echo.com/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "application/octet-stream"), data = as.raw(1L), response = c("Date", "Server"), timeout = 3000))
 test_class("errorValue", ncurl("http")$data)
-test_class("recvAio", haio <- ncurl_aio("http://example.com/"))
+test_class("recvAio", haio <- ncurl_aio("http://www.cam.ac.uk/", timeout = 3000L))
 test_true(is_aio(haio))
 test_type("integer", call_aio(haio)$status)
-test_class("ncurlAio", haio <- ncurl_aio("https://example.com/", convert = FALSE, response = "server"))
+test_class("ncurlAio", haio <- ncurl_aio("https://www.cam.ac.uk/", convert = FALSE, response = "server", timeout = 3000L))
 test_notnull(haio$status)
 if (call_aio(haio)$status == 200L) test_notnull(haio$headers)
 test_class("ncurlAio", put1 <- ncurl_aio("https://postman-echo.com/put", method = "PUT", headers = c(`Content-Type` = "text/plain", Authorization = "Bearer token"), data = "test", response = c("Date", "server"), timeout = 3000L))
@@ -444,6 +452,7 @@ test_type("integer", call_aio_(put1)$status)
 if (put1$status == 200L) test_notnull(put1$headers)
 if (put1$status == 200L) test_notnull(put1$data)
 test_null(stop_aio(put1))
+test_false(stop_request(put1))
 test_class("ncurlAio", haio <- ncurl_aio("https://i.i"))
 test_class("errorValue", call_aio(haio)$data)
 test_print(haio$data)
@@ -462,7 +471,7 @@ if (is_ncurl_session(sess)) test_zero(close(sess))
 if (is_ncurl_session(sess)) test_equal(transact(sess)$data, 7L)
 test_class("errorValue", suppressWarnings(ncurl_session("https://i")))
 test_error(ncurl_aio("https://", tls = "wrong"), "valid TLS")
-test_error(ncurl("https://www.example.com/", tls = "wrong"), "valid TLS")
+test_error(ncurl("https://www.cam.ac.uk/", tls = "wrong"), "valid TLS")
 test_type("externalptr", etls <- tls_config())
 test_error(stream(dial = "wss://127.0.0.1:5555", textframes = TRUE, tls = etls))
 test_error(stream(dial = "wss://127.0.0.1:5555"))
@@ -476,10 +485,11 @@ test_error(stream(), "specify a URL")
 
 test_type("character", ver <- nng_version())
 test_equal(length(ver), 2L)
-test_equal(nng_error(8L), "8 | Try again")
+test_equal(nng_error(5L), "5 | Timed out")
+test_equal(nng_error(8), "8 | Try again")
 test_true(is_nul_byte(as.raw(0L)))
-test_true(!is_nul_byte(NULL))
-test_true(!is_error_value(1L))
+test_false(is_nul_byte(NULL))
+test_false(is_error_value(1L))
 test_error(messenger("invalidURL"), "argument")
 test_type("raw", md5 <- nanonext:::md5_object("secret base"))
 test_equal(length(md5), 32L)
@@ -487,6 +497,7 @@ test_type("double", mclock())
 test_null(msleep(1L))
 test_null(msleep(1))
 test_null(msleep("a"))
+test_null(msleep(-1L))
 test_type("character", urlp <- parse_url("://"))
 test_equal(length(urlp), 10L)
 test_true(all(nzchar(parse_url("wss://use:r@[::1]/path?q=1#name"))))
@@ -540,10 +551,10 @@ test_error(context(fakesock), "valid Socket")
 test_error(.context(fakesock), "valid Socket")
 test_error(stat(fakesock, "pipes"), "valid Socket")
 test_error(close(fakesock), "valid Socket")
-test_true(!.unresolved(fakesock))
+test_false(.unresolved(fakesock))
 fakectx <- `class<-`("test", "nanoContext")
-test_true(!unresolved(fakectx))
-test_true(!.unresolved(fakectx))
+test_false(unresolved(fakectx))
+test_false(.unresolved(fakectx))
 test_error(request(fakectx, data = "test"), "valid Context")
 test_error(subscribe(fakectx, NULL), "valid")
 test_error(close(fakectx), "valid Context")
@@ -568,7 +579,7 @@ fakelist <- `class<-`("test", "nanoListener")
 test_error(start(fakelist), "valid Listener")
 test_error(close(fakelist), "valid Listener")
 unres <- `class<-`(NA, "unresolvedValue")
-test_true(!unresolved(unres))
+test_false(unresolved(unres))
 test_print(unres)
 test_type("logical", unres <- unresolved(list("a", "b")))
 test_equal(length(unres), 1L)
@@ -580,7 +591,9 @@ test_error(collect_aio_("a"), "object is not an Aio or list of Aios")
 test_error(collect_aio_(list("a")), "object is not an Aio or list of Aios")
 test_error(collect_aio(list(fakesock)), "object is not an Aio or list of Aios")
 test_null(stop_aio("a"))
+test_false(stop_request("a"))
 test_null(stop_aio(list("a")))
+test_false(stop_request(list("a")))
 test_null(.keep(NULL, new.env()))
 test_null(.keep(new.env(), new.env()))
 
@@ -602,10 +615,10 @@ test_identical(names(cert), c("server", "client"))
 test_type("externalptr", tls <- tls_config(client = cert$client))
 test_class("tlsConfig", tls)
 test_print(tls)
-test_class("errorValue", ncurl("https://www.example.com/", tls = tls)$status)
-test_class("errorValue", call_aio(ncurl_aio("https://www.example.com/", tls = tls))$data)
-test_error(ncurl_session("https://www.example.com/", tls = cert$client), "not a valid TLS")
-sess <- ncurl_session("https://www.example.com/", tls = tls)
+test_class("errorValue", ncurl("https://www.cam.ac.uk/", tls = tls)$status)
+test_class("errorValue", call_aio(ncurl_aio("https://www.cam.ac.uk/", tls = tls))$data)
+test_error(ncurl_session("https://www.cam.ac.uk/", tls = cert$client), "not a valid TLS")
+sess <- ncurl_session("https://www.cam.ac.uk/", tls = tls)
 test_true(is_ncurl_session(sess) || is_error_value(sess))
 if (is_ncurl_session(sess)) test_class("errorValue", transact(sess)[["headers"]])
 test_type("externalptr", s <- socket(listen = "tls+tcp://127.0.0.1:5556", tls = tls_config(server = cert$server)))
@@ -625,20 +638,20 @@ if (promises) test_true(promises::is.promise(promises::then(r2, identity, identi
 if (promises) test_true(promises::is.promising(call_aio(r)))
 if (promises) test_type("integer", send(s1, "promises test2\n", block = 500L))
 if (promises) test_true(promises::is.promise(promises::then(call_aio(recv_aio(s, timeout = 500L)), identity, identity)))
-if (promises) test_true(is_aio(n <- ncurl_aio("https://postman-echo.com/get")))
+if (promises) test_true(is_aio(n <- ncurl_aio("https://www.cam.ac.uk/", timeout = 3000L)))
 if (promises) test_true(promises::is.promise(promises::then(n, identity, identity)))
 if (promises) test_true(promises::is.promising(call_aio(n)))
-if (promises) test_true(promises::is.promise(promises::as.promise(call_aio(ncurl_aio("https://postman-echo.com/get")))))
-if (promises) later::run_now(1L)
+if (promises) test_true(promises::is.promise(promises::as.promise(call_aio(ncurl_aio("https://www.cam.ac.uk/", timeout = 3000L)))))
+if (promises) { later::run_now(1L); later::run_now() }
 if (promises) test_zero(close(s1))
 if (promises) test_zero(close(s))
-if (promises) later::run_now(1L)
+if (promises) { later::run_now(1L); later::run_now() }
 test_type("character", ip_addr())
 test_type("character", names(ip_addr()))
 test_null(write_stdout(""))
 test_true(.interrupt())
-test_true(!.interrupt(FALSE))
-test_true(!identical(get0(".Random.seed"), {.advance(); .Random.seed}))
+test_false(.interrupt(FALSE))
+test_false(identical(get0(".Random.seed"), {.advance(); .Random.seed}))
 test_type("integer", .Call(nanonext:::rnng_traverse_precious))
 
 if (!interactive() && Sys.getenv("NOT_CRAN") == "true") {
@@ -648,7 +661,7 @@ if (!interactive() && Sys.getenv("NOT_CRAN") == "true") {
   test_zero(pipe_notify(reader, cv, remove = TRUE, flag = TRUE))
   test_true(is_aio(r <- recv_aio(reader, mode = "raw", cv = cv)))
   close(f)
-  test_true(!wait(cv))
+  test_false(wait(cv))
   test_zero(close(reader))
   test_equal(collect_aio(r), 7L)
 }
